/***************************************************************************
 *   Copyright (C) 2006 by AST   *
 *   tsyplyaev@gmail.com   *
 *   ICQ: 279-533-134                          *
 *   This is a part of work done by AST.       *
 *   If you want to use it, please contact me. *
 ***************************************************************************/

//***************************************************
#include "Engine.h"
//***************************************************

namespace ASTEngine {

#define ENGINE_VERSION_NUMBER 0.2.8
#define ENGINE_VERSION_STRING "0.2.8"

Engine *engine;

//---------------------------------------------------------------------------
//Desc:    creates new ASTEngine
//Params:  -
//Returns: pointer to new ASTEngine
//---------------------------------------------------------------------------
Engine *Engine::create() {
	if(engine) { return engine; };
	
	engine = new Engine();

	Log::create();
	Log::write("AST engine "ENGINE_VERSION_STRING"\n");

	engine->rc = false;
	engine->ec = false;

	engine->config = Config::create("ast_conf.txt");
		
	int w = engine->config->getInt("window_width");
	int h = engine->config->getInt("window_height");
	int b = engine->config->getInt("window_bpp");
	int z = engine->config->getInt("window_zdepth");
	bool f = engine->config->getBool("window_fullscreen");

	engine->windowSystem  = WindowSystem::create(w, h, b, z, f);
	engine->glSystem   = GLSystem::create();
	engine->alSystem   = ALSystem::create();
	engine->ilSystem   = ILSystem::create();
	engine->physSystem = PhysSystem::create();
	engine->cash       = Cash::create();
	

	engine->glSystem->requireExtension("GL_ARB_vertex_shader");
	engine->glSystem->requireExtension("GL_ARB_fragment_shader");
	engine->glSystem->requireExtension("GL_ARB_shader_objects");
	engine->glSystem->requireExtension("GL_ARB_shading_language_100");

	engine->glSystem->requireExtension("GL_ARB_vertex_buffer_object");
	engine->glSystem->requireExtension("GL_EXT_framebuffer_object");

	engine->glSystem->requireExtension("GL_ARB_occlusion_query");

	engine->glSystem->requireExtension("GL_EXT_texture_filter_anisotropic");
	
	engine->running = true;

	return engine;
};

//---------------------------------------------------------------------------
//Desc:    returns existing ASTEngine
//Params:  -
//Returns: pointer to existing ASTEngine
//---------------------------------------------------------------------------
Engine* Engine::get() {
	return engine;
};

//---------------------------------------------------------------------------
//Desc:    ASTEngine destructor
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
Engine::~Engine()  {
	if(GUI::get()) delete GUI::get();
	if(Scene::get()) delete Scene::get();
	
	delete physSystem;
	delete ilSystem;
	delete alSystem;
	delete glSystem;

	delete windowSystem;
};

//---------------------------------------------------------------------------
//Desc:    engines main loop
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
void Engine::mainLoop() {
	windowSystem->update();

	while(running) {
		windowSystem->update();
		physSystem->update(windowSystem->getDTime());

		if(ec) events_callback();

		glSystem->clear(GLSystem::COLOR_BUFFER | GLSystem::DEPTH_BUFFER | GLSystem::STENCIL_BUFFER);
		
		if(Scene::get()) Scene::get()->draw();
		if(rc) render_callback();
		if(GUI::get()) GUI::get()->draw();

		glSystem->flush();

		windowSystem->swapBuffers();
	};
};

//---------------------------------------------------------------------------
//Desc:    sets render callback
//Params:  callback - pointer to render function
//Returns: -
//---------------------------------------------------------------------------
void Engine::renderCallback(EngineCallback callback) {
	render_callback = callback;
	rc = true;
};

//---------------------------------------------------------------------------
//Desc:    sets events callback
//Params:  callback - pointer to events function
//Returns: -
//---------------------------------------------------------------------------
void Engine::eventsCallback(EngineCallback callback) {
	events_callback = callback;
	ec = true;
};

//---------------------------------------------------------------------------
//Desc:    exits the main loop
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
void Engine::quit() {
	running = false;
};

};
