/***************************************************************************
 *   Copyright (C) 2006 by AST   *
 *   tsyplyaev@gmail.com   *
 *   ICQ: 279-533-134                          *
 *   This is a part of work done by AST.       *
 *   If you want to use it, please contact me. *
 ***************************************************************************/

//**************************************
#include "GUI.h"
#include "ILSystem.h"
#include "WindowSystem.h"
//**************************************

namespace ASTEngine {

GUI *gui;

//**************************************************************************
//GUI class
//**************************************************************************
//---------------------------------------------------------------------------
//Desc:    creates new GUI
//Params:  -
//Returns: pointer to new GUI
//---------------------------------------------------------------------------
GUI *GUI::create(const String &folder) {
	if(gui) return gui;

	gui = new GUI();
	
	gui->alpha = 1.0;

	gui->windowTex = GLTexture::create2d(folder + "/window.png");
	gui->windowTex->setWrap(GLTexture::CLAMP_TO_EDGE);

	gui->buttonTex = GLTexture::create2d(folder + "/button.png");
	gui->buttonTex->setWrap(GLTexture::CLAMP_TO_EDGE);

	gui->buttonFocusedTex = GLTexture::create2d(folder + "/button_pressed.png");
	gui->buttonFocusedTex->setWrap(GLTexture::CLAMP_TO_EDGE);

	gui->checkBoxTex = GLTexture::create2d(folder + "/check_box.png");
	gui->checkBoxTex->setWrap(GLTexture::CLAMP_TO_EDGE);

	gui->checkBoxCheckedTex = GLTexture::create2d(folder + "/check_box_checked.png");
	gui->checkBoxCheckedTex->setWrap(GLTexture::CLAMP_TO_EDGE);

	gui->hSliderTex = GLTexture::create2d(folder + "/hslider.png");
	gui->hSliderTex->setWrap(GLTexture::CLAMP_TO_EDGE);
	
	gui->font = new Font(folder + "/font.png");

	return gui;
};

//---------------------------------------------------------------------------
//Desc:    returns existing GUI
//Params:  -
//Returns: pointer to the GUI
//---------------------------------------------------------------------------
GUI *GUI::get() {
	return gui;
};

//---------------------------------------------------------------------------
//Desc:    GUI destructor
//Params:  -
//Returns: pointer to the GUI
//---------------------------------------------------------------------------
GUI::~GUI() {
	delete windowTex;
	delete buttonTex;
	delete buttonFocusedTex;
	delete checkBoxTex;
	delete checkBoxCheckedTex;
	delete hSliderTex;
	
	delete font;
};

//---------------------------------------------------------------------------
//Desc:    adds Widget to the GUI
//Params:  widget - pointer to the Widget to add
//Returns: -
//---------------------------------------------------------------------------
void GUI::addWidget(Widget *widget) {
	widget->setActive(true);
	widgets.push_back(widget);
};

//---------------------------------------------------------------------------
//Desc:    removes Widget from the GUI
//Params:  widget - pointer to the Widget to remove
//Returns: -
//---------------------------------------------------------------------------
void GUI::removeWidget(Widget *widget) {
	for(int i = 0; i < widgets.size(); i++) {
		if(widgets[i] == widget)
			widgets[i] = NULL;
	};
};

//---------------------------------------------------------------------------
//Desc:    draw all GUI widgets
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
void GUI::draw() {
	for(int i = 0; i < widgets.size(); i++) {
		if(widgets[i]) {
			widgets[i]->draw();
		};
	};
};




//**************************************************************************
//Widget class
//**************************************************************************
//---------------------------------------------------------------------------
//Desc:    is Widget focused
//Params:  -
//Returns: true - if focused
//---------------------------------------------------------------------------
bool Widget::isFocused() {
	if(WindowSystem::get()->isMouseGrabed())
		return false;

	int mx = WindowSystem::get()->getMouseX();
	int my = WindowSystem::get()->getMouseY();

	return ((getX() < mx) && (getX() + width > mx) &&
		(getY() < my) && (getY() + height > my)) && active;
};

//---------------------------------------------------------------------------
//Desc:    is Widget pressed
//Params:  -
//Returns: true - if pressed
//---------------------------------------------------------------------------
bool Widget::isPressed() {
	return (isFocused() && WindowSystem::get()->isMouseButtonPressed(WindowSystem::MOUSE_LEFT_BUTTON) && active);
};

//---------------------------------------------------------------------------
//Desc:    was Widget pressed
//Params:  -
//Returns: true - if it was released and now pressed
//---------------------------------------------------------------------------
bool Widget::wasPressed() {
	return isFocused() && WindowSystem::get()->wasMouseButtonPressed(WindowSystem::MOUSE_LEFT_BUTTON);
};

//---------------------------------------------------------------------------
//Desc:    was Widget released
//Params:  -
//Returns: true - if it was pressed and now released
//---------------------------------------------------------------------------
bool Widget::wasReleased() {
	return isFocused() && WindowSystem::get()->wasMouseButtonReleased(WindowSystem::MOUSE_LEFT_BUTTON);
};

//---------------------------------------------------------------------------
//Desc:    draw Widget
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
void Widget::draw() {
};

//---------------------------------------------------------------------------
//Desc:    adds Widget to the widgets list
//Params:  widget - pointer to widget to add
//Returns: -
//---------------------------------------------------------------------------
void Widget::addWidget(Widget *widget) {
	widget->parent = this;
	children.push_back(widget);
};

//---------------------------------------------------------------------------
//Desc:    removes Widget from the widgets list
//Params:  widget - pointer to widget to remove
//Returns: -
//---------------------------------------------------------------------------
void Widget::removeWidget(Widget *widget) {
	for(int i = 0; i < children.size(); i++) {
		if(children[i] == widget)
			children[i]->parent = NULL;
			children[i] = NULL;
	};
};




//**************************************************************************
//WidgetButton class
//**************************************************************************
//---------------------------------------------------------------------------
//Desc:    WidgetButton constructor
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
WidgetButton::WidgetButton(const String &text) {
	this->text = text;
	
	width = text.size() * 20;
	height = 30;
	
	posX = 0;
	posY = 0;

	parent = NULL;

	active = false;
};

//---------------------------------------------------------------------------
//Desc:    WidgetButton destructor
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
WidgetButton::~WidgetButton() {
};

//---------------------------------------------------------------------------
//Desc:    draw WidgetButton
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
void WidgetButton::draw() {
	if(!active)	return;

	GLSystem::get()->enableBlending(GLSystem::ONE, GLSystem::ONE_MINUS_SRC_ALPHA);
	
	if(isFocused()) {
		GUI::get()->buttonFocusedTex->set(0);
	} else {
		GUI::get()->buttonTex->set(0);
	};

	GLSystem::get()->setColor(Vec4(1, 1, 1, 1) * GUI::get()->alpha);
	
	GLSystem::get()->enable2d(false);
	GLSystem::get()->drawRect(getX(), getY(), getX() + width, getY() + height, 0, 0, 1, 1);
	GLSystem::get()->enable3d();
	
	GUI::get()->buttonTex->unset(0);
	GLSystem::get()->disableBlending();
	
	int fSize = height*0.5;
	int tPosX = (width - text.length()*fSize*0.8)*0.5 + getX();
	int tPosY = (height - fSize)*0.5 + getY();
	
	GUI::get()->font->print(tPosX, tPosY, fSize, text, Vec3( 1, 1, 1 ), GUI::get()->alpha);

	for(int w = 0; w < children.size(); w++) {
		if(children[w]) {
			children[w]->setActive(active);
			children[w]->draw();
		};
	};
};





//**************************************************************************
//WidgetCheckBox class
//**************************************************************************
//---------------------------------------------------------------------------
//Desc:    WidgetCheckBox constructor
//Params:  size - size in pixels, x,y - position
//Returns: -
//---------------------------------------------------------------------------
WidgetCheckBox::WidgetCheckBox(const String &text) {
	this->text = text;
	
	width = 15;
	height = 15;

	posX = 0;
	posY = 0;

	parent = NULL;
	
	checked = false;
	active = false;
};

//---------------------------------------------------------------------------
//Desc:    WidgetCheckBox destructor
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
WidgetCheckBox::~WidgetCheckBox() {
};

//---------------------------------------------------------------------------
//Desc:    draw WidgetCheckBox
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
void WidgetCheckBox::draw() {
	if(!active) return;

	GLSystem::get()->enableBlending(GLSystem::ONE, GLSystem::ONE_MINUS_SRC_ALPHA);
	
	if(wasReleased()) toggleCheck();

	if(!checked) {
		GUI::get()->checkBoxTex->set(0);
	} else {
		GUI::get()->checkBoxCheckedTex->set(0);
	};

	GLSystem::get()->setColor(Vec4(1, 1, 1, 1) * GUI::get()->alpha);
	
	GLSystem::get()->enable2d(false);
	GLSystem::get()->drawRect(getX(), getY(), getX() + width, getY() + height, 0, 0, 1, 1);
	GLSystem::get()->enable3d();
	
	GUI::get()->checkBoxTex->unset(0);
	GLSystem::get()->disableBlending();

	for(int w = 0; w < children.size(); w++) {
		if(children[w]) {
			children[w]->setActive(active);
			children[w]->draw();
		};
	};

	int fSize = height*0.8;
	int tPosX = width + getX();
	int tPosY = getY();
	GUI::get()->font->print(tPosX, tPosY, fSize, text, Vec3( 1, 1, 1 ), GUI::get()->alpha);
};




//**************************************************************************
//WidgetRadioGroup class
//**************************************************************************
//---------------------------------------------------------------------------
//Desc:    WidgetRadioGroup constructor
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
WidgetRadioGroup::WidgetRadioGroup() {
	posX = 0;
	posY = 0;
	
	parent = NULL;
};

WidgetRadioGroup::~WidgetRadioGroup() {
};

void WidgetRadioGroup::addCheckBox(WidgetCheckBox *cbox) {
	cbox->parent = this;
	checkBoxes.push_back(cbox);
};

void WidgetRadioGroup::removeCheckBox(WidgetCheckBox *cbox) {
	for(int i = 0; i < checkBoxes.size(); i++) {
		if(checkBoxes[i] == cbox)
			checkBoxes[i]->parent = NULL;
			checkBoxes[i] = NULL;
	};
};

void WidgetRadioGroup::draw() {
	if(!active) return;

	int c = -1;
	for(int w = 0; w < checkBoxes.size(); w++) {
		if(checkBoxes[w]) {
			if(checkBoxes[w]->isChecked() && checkBoxes[w]->wasPressed()) {
				checkBoxes[w]->setChecked(false);
				c = w;
				break;
			} else if(!checkBoxes[w]->isChecked() && checkBoxes[w]->wasPressed()) {
				checkBoxes[w]->setChecked(false);
				c = w;
				break;
			} else if(checkBoxes[w]->isChecked()) {
				c = w;
			};
		};
	};

	for(int w = 0; w < checkBoxes.size(); w++) {
		if(c == -1) break;
		if(checkBoxes[w] && w != c) {
			checkBoxes[w]->setChecked(false);
		};
	};

	for(int w = 0; w < checkBoxes.size(); w++) {
		if(checkBoxes[w]) {
			checkBoxes[w]->setActive(true);
			checkBoxes[w]->draw();
		};
	};
};




//**************************************************************************
//WidgetLabel class
//**************************************************************************
//---------------------------------------------------------------------------
//Desc:    WidgetLabel constructor
//Params:  text - text string, size - size in pixels, x,y - position
//Returns: -
//---------------------------------------------------------------------------
WidgetLabel::WidgetLabel(const String &text) {
	this->text = text;
	
	width = 15 * text.size();
	height = 15;

	posX = 0;
	posY = 0;

	parent = NULL;

	active = false;
};

//---------------------------------------------------------------------------
//Desc:    WidgetLabel destructor
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
WidgetLabel::~WidgetLabel() {
};

//---------------------------------------------------------------------------
//Desc:    draw WidgetLabel
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
void WidgetLabel::draw() {
	if(!active) return;

	GUI::get()->font->print(getX(), getY(), getHeight(), text, Vec3( 1, 1, 1 ), GUI::get()->alpha);
};




//**************************************************************************
//WidgetHSlider class
//**************************************************************************
//---------------------------------------------------------------------------
//Desc:    WidgetHSlider costructor
//Params:  w,h - size, x,y - position
//Returns: -
//---------------------------------------------------------------------------
WidgetHSlider::WidgetHSlider() {
	width = 100;
	height = 10;

	posX = 0;
	posY = 0;

	parent = NULL;

	setValue(10);

	active = false;
};

//---------------------------------------------------------------------------
//Desc:    WidgetHSlider destructor
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
WidgetHSlider::~WidgetHSlider() {
};

//---------------------------------------------------------------------------
//Desc:    draw WidgetHSlider
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
void WidgetHSlider::draw() {
	if(pos > width) pos = width;
	if(pos < 0) pos = 0;
	
	if(!active) return;

	GLSystem::get()->enableBlending(GLSystem::ONE, GLSystem::ONE_MINUS_SRC_ALPHA);
	
	if(isPressed()) {
		pos = WindowSystem::get()->getMouseX() - getX();
		value = (int)(100.0 * (float)pos/(float)width);
	};
	

	GUI::get()->hSliderTex->set(0);

	GLSystem::get()->setColor(Vec4(1, 1, 1, 1) * GUI::get()->alpha);
	
	GLSystem::get()->enable2d(false);
	GLSystem::get()->drawRect(getX(), getY(), getX() + width, getY() + height, 0, 0, 1, 1);
	
	GUI::get()->checkBoxTex->set(0);

	GLSystem::get()->drawRect(getX() + pos - height*0.6, 
		getY() - height*0.1, 
		getX() + pos + height*0.6, 
		getY() + height*1.1, 
		0, 0, 1, 1);
	GLSystem::get()->enable3d();
	
	GUI::get()->buttonTex->unset(0);
	GLSystem::get()->disableBlending();

	for(int w = 0; w < children.size(); w++) {
		if(children[w]) {
			children[w]->setActive(active);
			children[w]->draw();
		};
	};
};



//**************************************************************************
//WidgetWindow class
//**************************************************************************
//---------------------------------------------------------------------------
//Desc:    WidgetWindow costructor
//Params:  w,h - size, x,y - position
//Returns: -
//---------------------------------------------------------------------------
WidgetWindow::WidgetWindow() {
	width = 400;
	height = 300;
	
	posX = 0;
	posY = 0;

	parent = NULL;

	active = false;
};

//---------------------------------------------------------------------------
//Desc:    WidgetWindow destructor
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
WidgetWindow::~WidgetWindow() {
};

//---------------------------------------------------------------------------
//Desc:    draw WidgetWindow
//Params:  -
//Returns: -
//---------------------------------------------------------------------------
void WidgetWindow::draw() {
	static bool state = false;
	
	bool p = false;
	if(isPressed()) {
		p = true;
		for(int w = 0; w < children.size(); w++) {
			if(children[w]->isFocused()) {
				p = false;
			};
		};
	};

	if(!state) { state = wasPressed() && p; };
	if(state) { state = !wasReleased(); };

	if(isPressed()) {
		posX += WindowSystem::get()->getMouseDX();
		posY += WindowSystem::get()->getMouseDY();
	};
	
	if(active) {
		GLSystem::get()->enableBlending( GLSystem::ONE, GLSystem::ONE_MINUS_SRC_ALPHA );
		GUI::get()->windowTex->set(0);
	
		GLSystem::get()->setColor(Vec4(1, 1, 1, 1) * GUI::get()->alpha);
		
		GLSystem::get()->enable2d(false);
		GLSystem::get()->drawRect(getX(), getY(), getX() + width, getY() + height, 0, 0, 1, 1);
		GLSystem::get()->enable3d();
		
		GUI::get()->windowTex->unset(0);
		GLSystem::get()->disableBlending();
	};

	for(int w = 0; w < children.size(); w++) {
		if(children[w]) {
			children[w]->setActive(active);
			children[w]->draw();
		};
	};
};

};

