/***************************************************************************
 *   Copyright (C) 2006 by AST   *
 *   tsyplyaev@gmail.com   *
 *   ICQ: 279-533-134                          *
 *   This is a part of work done by AST.       *
 *   If you want to use it, please contact me. *
 ***************************************************************************/

#pragma once

//***************************************************
#include "IncludesAndLibs.h"
#include "Config.h"
//***************************************************************************
#include "String.h"
//***************************************************
#ifdef WIN32
#include <windows.h>
#endif
//***************************************************

namespace ASTEngine {

//---------------------------------------------------------------------------
//Desc: Engine`s main window and input system. Created one time
//---------------------------------------------------------------------------
class WindowSystem {
public:
	//---------------------------------------------------------------------------
	//Desc: Mouse buttons enum
	//---------------------------------------------------------------------------
	enum MouseButton {
		MOUSE_LEFT_BUTTON = 0,
		MOUSE_RIGHT_BUTTON = 1,
	};

	//---------------------------------------------------------------------------
	//Desc: Keyboard keys enum
	//---------------------------------------------------------------------------
	enum Key {
		KEY_ESC = VK_ESCAPE, 
		KEY_SPACE = VK_SPACE,

		KEY_UP = VK_UP,   
		KEY_DOWN = VK_DOWN,
		KEY_LEFT = VK_LEFT, 
		KEY_RIGHT = VK_RIGHT,

		KEY_0  = VK_NUMPAD0, 
		KEY_1,
		KEY_2, KEY_3,
		KEY_4, KEY_5, 
		KEY_6, KEY_7 , 
		KEY_8, KEY_9,

		KEY_A = 65, 
		KEY_B,
		KEY_C, KEY_D,
		KEY_E, KEY_F,
		KEY_G, KEY_H,
		KEY_I, KEY_J,
		KEY_K, KEY_L,
		KEY_M, KEY_N,
		KEY_O, KEY_P,
		KEY_Q, KEY_R, 
		KEY_S, KEY_T, 
		KEY_U, KEY_V, 
		KEY_W, KEY_X, 
		KEY_Y, KEY_Z,
	};

public:
	static WindowSystem *create(int width, int height, int bpp, int zdepth, bool fullscreen);
	static WindowSystem *get();
	~WindowSystem();

	void setTitle(const String &title);

	void updateTimer();
	void swapBuffers();

	void update();

	int getWidth() { return width; };
	int getHeight() { return height; };
		
	int getDTime() { return dTime; };
	int getETime() { return eTime; };
	int getTime();

	bool isMouseMoved() { return mousing; };
	int getMouseX() { return mouseX; };
	int getMouseY() { return mouseY; };
	int getMouseDX() { return mouseX - oldMouseX; };
	int getMouseDY() { return mouseY - oldMouseY; };
	
	void setMousePos(int x, int y);
	
	void showCursor(bool show);
	void toggleShowCursor() { showCursor(!cursorVisible); };
	bool isCursorVisible() { return cursorVisible; };
	
	void grabMouse(bool grab);
	void toggleGrabMouse() { grabMouse(!mouseGrabed); };
	bool isMouseGrabed() { return mouseGrabed; };

	bool isMouseButtonPressed(MouseButton mb);
	bool wasMouseButtonPressed(MouseButton mb);
	bool wasMouseButtonReleased(MouseButton mb);
	
	bool isKeyPressed(Key key);
	bool wasKeyPressed(Key key);
	bool wasKeyReleased(Key key);

private:
	HDC hDC;		
	HGLRC hRC;		
	HWND hWnd;		
	HINSTANCE hInstance;	

public:
	int width, height, bpp, zdepth;
	bool fullscreen;
	
	int eTime, dTime;
	int mouseX, mouseY;
	int oldMouseX, oldMouseY;

	bool cursorVisible;
	bool mouseGrabed;

	int mx, my;
	bool mousing;

	bool mouseButtons[3];
	bool oldMouseButtons[3];

	bool keys[315];
	bool oldKeys[315];
};

};



