#include "AST_engine_src/Engine.h"
#include "Main.h"

using namespace ASTEngine;

#ifdef SPONZA_DEMO
	//objects
	ObjectMesh *sponza;
	ObjectMesh *box[5];
	ObjectMesh *sphere[5];
	ObjectMesh *cylinder[5];
	//ObjectSkinnedMesh *chammy;
	CameraFree *camera;
	LightOmni *omniLight;
	
	ParticleSystem *particlesPink;
	ParticleSystem *particlesYellow;

	//GUI widgets
	WidgetWindow *window;
	WidgetLabel *label;
	WidgetCheckBox *cb[4], *cbSP, *cbHDR, *cbWire;
	WidgetRadioGroup *group;
	WidgetButton *exitButton;
	WidgetLabel *fpsLabel;

	//------------------------------------------------------------
	void init() {
		//initializing loading screen
		LoadingScreen *lscreen = LoadingScreen::create("data/textures/logos/background.jpg");
		lscreen->show();

		//initialize GUI
		GUI::create("data/textures/gui");
		GUI::get()->setAlpha(0.8);

		GUI::get()->addWidget(fpsLabel = new WidgetLabel("FPS: "));
		fpsLabel->setPosition(20, 40);
		
		GUI::get()->addWidget(window = new WidgetWindow());
		window->setPosition(500, 300);
		window->setActive(false);
		
		window->addWidget(label = new WidgetLabel("Settings"));
		label->setPosition(10, 10);
		window->addWidget(group = new WidgetRadioGroup());
		group->setPosition(10, 10);

		window->addWidget(cbSP = new WidgetCheckBox("specular"));
		cbSP->setPosition(10, 180);
		cbSP->setChecked(Config::get()->getInt("light_specular"));

		window->addWidget(cbHDR = new WidgetCheckBox("HDR bloom"));
		cbHDR->setChecked(Config::get()->getBool("effect_hdr"));
		cbHDR->setPosition(10, 210);

		window->addWidget(cbWire = new WidgetCheckBox("wireframe"));
		cbWire->setChecked(Config::get()->getBool("debug_wireframe"));
		cbWire->setPosition(10, 240);

		group->addCheckBox(cb[0] = new WidgetCheckBox("no shadows"));
		cb[0]->setPosition(10, 30);
		group->addCheckBox(cb[1] = new WidgetCheckBox("sm shadows"));
		cb[1]->setPosition(10, 60);
		group->addCheckBox(cb[2] = new WidgetCheckBox("sm shadows with PCF 2x2"));
		cb[2]->setPosition(10, 90);
		group->addCheckBox(cb[3] = new WidgetCheckBox("sm shadows with PCF 3x3"));
		cb[3]->setPosition(10, 120);
		cb[Config::get()->getInt("light_shadowtype")]->setChecked(true);

		window->addWidget(exitButton = new WidgetButton("Exit"));
		exitButton->setPosition(300, 250);

		//initializing scene
		Scene::create();

		
		//chammy = new ObjectSkinnedMesh("chammy.asmdl");
		//chammy->setMaterial("*", "usor.amtr");
		//chammy->setTransform(Mat4::translate(Vec3(10, 10, 10)));
		//Scene::get()->addObject(chammy);

		sponza = new ObjectMesh("sponza.amdl");
		sponza->setMaterialList("sponza.amtrlst");
		sponza->setPhysicsStaticMesh();
		sponza->setTransform(Mat4::translate(Vec3(0, -10, 0)));
		Scene::get()->addObject(sponza);
	

		for(int i = 0; i < 5; i++) {
			box[i] = new ObjectMesh("cube.amdl");
			box[i]->setMaterial("*", "grid.amtr");
			box[i]->setPhysicsBox(Vec3(10, 10, 10), 10);
			box[i]->setTransform(Mat4::translate(Vec3(-10-i*2, i*20 + 10, i - 10)));
			box[i]->setImpactSound("impact.ogg");

			Scene::get()->addObject(box[i]);
		};

		for(int i = 0; i < 5; i++) {
			sphere[i] = new ObjectMesh("sphere.amdl");
			sphere[i]->setMaterial("*", "grid.amtr");
			sphere[i]->setPhysicsSphere(Vec3(5,5,5), 10);
			sphere[i]->setTransform(Mat4::translate(Vec3(10+i*2, i*20 + 10, i - 10)));
			sphere[i]->setImpactSound("impact.ogg");

			Scene::get()->addObject(sphere[i]);
		};
	
		for(int i = 0; i < 5; i++) {
			cylinder[i] = new ObjectMesh("torus.amdl");
			cylinder[i]->setMaterial("*", "grid.amtr");
			cylinder[i]->setPhysicsConvexHull(10);
			cylinder[i]->setTransform(Mat4::translate(Vec3(20+i*2, i*20 + 20, i - 10)));
			cylinder[i]->setImpactSound("impact.ogg");

			Scene::get()->addObject(cylinder[i]);
		};
			
		camera = new CameraFree();
		camera->setPosition(Vec3(0, 20, 0));
		camera->setMaxVelocity(2000);
		camera->setPhysics(Vec3(5, 5, 5), 1.0);
		camera->setFOV(60);
		Scene::get()->setCamera(camera);
		
		omniLight = new LightOmni();
		omniLight->setColor(Vec3(1, 1, 1));
		omniLight->setPosition(Vec3(0, 60, 0));
		omniLight->setRadius(200);
		Scene::get()->addLight(omniLight);

		particlesPink = new ParticleSystem("data/textures/smoke.png", 50);
		particlesPink->setPosition(Vec3(60, 60, 0));
		particlesPink->setColor(Vec3(1, 0.5, 1));
		particlesPink->setForce(Vec3(-0.5, 1, -0.7));
		particlesPink->setVelocity(Vec3(-10, 0, 0));
		particlesPink->setParticleLifeTime(10000);
		particlesPink->setDispersion(0.1);
		Scene::get()->addParticleSystem(particlesPink);

		particlesYellow = new ParticleSystem("data/textures/smoke.png", 50);
		particlesYellow->setPosition(Vec3(-60, 60, 0));
		particlesYellow->setColor(Vec3(1, 1, 0.7));
		particlesYellow->setForce(Vec3(0.5, 1, 0.5));
		particlesYellow->setVelocity(Vec3(10, 0, 0));
		particlesYellow->setParticleLifeTime(10000);
		particlesYellow->setDispersion(0.1);
		Scene::get()->addParticleSystem(particlesYellow);

		Scene::get()->setGravity(Vec3(0, -9.8, 0));
		Scene::get()->setWater(1, 400);
		Scene::get()->setAmbient(Vec3(0.2, 0.2, 0.2));

		WindowSystem::get()->grabMouse(true);
	};

	//------------------------------------------------------------
	void render() {

	};

	float depth = 10;
	//------------------------------------------------------------
	void events() {
		//chammy->setFrame((int)(WindowSystem::get()->getETime() * 0.05)%500);

		if(WindowSystem::get()->wasKeyPressed(WindowSystem::KEY_ESC)) {
			window->toggleActive();
			WindowSystem::get()->toggleGrabMouse();
		};

		for(int i = 0; i < 4; i++) {
			if(cb[i]->wasPressed()) {
				Config::get()->setInt("light_shadowtype", i);
				Scene::get()->reloadShaders();
			};
		};

		if(cbSP->isPressed()) {
			Config::get()->setBool("light_specular", cbSP->isChecked());
			Scene::get()->reloadShaders();
		};

		if(cbHDR->isPressed()) {
			Config::get()->setBool("effect_hdr", cbHDR->isChecked());
		};
		
		if(cbWire->isPressed()) {
			Config::get()->setBool("debug_wireframe", cbWire->isChecked());
		};

		if(WindowSystem::get()->getDTime() > EPSILON)
			fpsLabel->setText("FPS: " + StringHelper::fromInt(1000/WindowSystem::get()->getDTime()));

		if(exitButton->isPressed()) {
			Engine::get()->quit();
		};

		if(WindowSystem::get()->isKeyPressed(WindowSystem::KEY_Z)) {
			Scene::get()->setWater(depth += 1, 500);
		};

		if(WindowSystem::get()->isKeyPressed(WindowSystem::KEY_X)) {
			Scene::get()->setWater(depth -= 1, 500);
		};
	};
#endif